module UserDecorator
  def logged_user?
    User.current.id == id
  end

  def name
    [lastname, firstname].join('')
  end

  # ---------- homepage helper ---------
  def homepage_name
    logged_user? ? real_name : full_name
  end

  # 关注数
  def follow_count
    User.watched_by(id).count
  end

  # 粉丝数
  def fan_count
    watchers.count
  end

  # 是否绑定邮箱
  def email_binded?
    mail.present?
  end

  # 学院的url标识
  def college_identifier

    Department.find_by_id(department_members.pluck(:department_id).first)&.identifier
  end

  # 是否能申请试用
  def can_apply_trial?
    return false if certification == 1

    apply = ApplyAction.order(created_at: :desc).find_by(user_id: id, container_type: 'TrialAuthorization')

    apply && !apply.status.zero?
  end

  # 是否已经签到
  def attendance_signed?
    attendance = Attendance.find_by(user_id: id)

    attendance && Util.days_between(Time.zone.now, attendance.created_at).zero?
  end

  # 明日签到金币
  def tomorrow_attendance_gold
    Attendance.find_by(user_id: id)&.next_gold
  end

  # ----------- 账号管理 -------------
  def authentication_status
    if authentication?
      'certified'
    elsif process_real_name_apply.present?
      'applying'
    else
      'uncertified'
    end
  end

  def professional_certification_status
    if professional_certification?
      'certified'
    elsif process_professional_apply.present?
      'applying'
    else
      'uncertified'
    end
  end

  def base_info_completed?
    user_columns = %i[nickname lastname]
    user_extension_columns = %i[gender location location_city identity school_id department]

    user_columns.all? { |column| public_send(column).present? } &&
      user_extension_columns.all? { |column| user_extension.send(column).present? }
  end

  def all_certified?
    authentication? && professional_certification?
  end
end